@@ -32,7 +32,8 @@ use strict;
 use Prima;
 use Prima::IntUtils;
 
-package ci;
+package 
+    ci;
 
 BEGIN {
 	eval 'use constant Grid           => 1 + MaxId;' unless exists $ci::{Grid};
@@ -40,7 +41,8 @@ BEGIN {
 	eval 'use constant IndentCellBack => 3 + MaxId;' unless exists $ci::{IndentCellBack};
 }
 
-package gsci;
+package 
+    gsci;
 
 use constant COL_INDEX => 0;
 use constant ROW_INDEX => 1;
@@ -58,13 +60,14 @@ use constant RECT      => 7,8,9,10;
 
 package Prima::AbstractGridViewer;
 use vars qw(@ISA);
-@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller);
+@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller Prima::ListBoxUtils);
 
 {
 my %RNT = (
 	%{Prima::Widget-> notification_types()},
 	DrawCell      => nt::Action,
 	GetRange      => nt::Action,
+	GetAlignment  => nt::Action,
 	Measure       => nt::Action,
 	SelectCell    => nt::Default,
 	SetExtent     => nt::Action,
@@ -101,6 +104,9 @@ sub profile_default
 		rows                    => 1,
 		topCell                 => 0,
 		scaleChildren           => 0,
+		scrollBarClass          => 'Prima::ScrollBar',
+		hScrollBarProfile       => {},
+		vScrollBarProfile       => {},
 		selectable              => 1,
 		vScroll                 => 1,
 		widgetClass             => wc::ListBox,
@@ -136,6 +142,7 @@ sub init
 	my %profile = $self-> SUPER::init(@_);
 	$self-> setup_indents;
 
+	$self->{$_} = $profile{$_} for qw(scrollBarClass hScrollBarProfile vScrollBarProfile);
 	$self-> $_( $profile{ $_}) for qw( 
 		allowChangeCellHeight allowChangeCellWidth
 		constantCellWidth constantCellHeight
@@ -173,16 +180,18 @@ sub draw_cells
 	my @selection = $self-> selection;
 	my @f = $self-> focused ? $self-> focusedCell : ( -1, -1);
 	$self-> push_event;
-	my ( $xsel, $ysel);
+	my @prelight    = @{$self->{prelight} // [-1,-1]};
+	my ( $xsel, $ysel, $xprelight, $yprelight);
 	my ( $clipV, $clipH) = ( $self-> {clipCells} == 1, $self-> {clipCells} == 2);
 	for ( @$cols) {
 		my ( $col, $xtype, $br, $x1, $x2, $X1, $X2) = @$_;
 		$canvas-> clipRect( $x1, $$active_area[1], $x2, $$active_area[3]) if $clipV;
-		$xsel = ( $col >= $selection[0] && $col <= $selection[2] ) ? 1 : 0;
+		$xsel = $col >= $selection[0] && $col <= $selection[2];
+		$xprelight = $col == $prelight[0];
 		for ( @$rows) {
 			my ( $row, $ytype, $br, $y1, $y2, $Y1, $Y2) = @$_;
-			$ysel = ( $row >= $selection[1] && $row <= $selection[3] ) ? 1 : 0 
-				if $xsel;
+			$ysel = $row >= $selection[1] && $row <= $selection[3] if $xsel;
+			$yprelight = $row == $prelight[1];
 			$canvas-> clipRect( $x1, $y1, $x2, $y2) if $clipH;
 			$notifier-> ( @notifyParms, 
 				$canvas, 
@@ -190,7 +199,8 @@ sub draw_cells
 				$x1, $y1, $x2, $y2,
 				$X1, $Y1, $X2, $Y2,
 				$xsel && $ysel,
-				( $col == $f[0] && $row == $f[1]) ? 1 : 0
+				( $col == $f[0] && $row == $f[1]) ? 1 : 0,
+				$xprelight && $yprelight
 			); 
 		}
 	}
@@ -205,8 +215,24 @@ sub draw_text_cells
 	for ( $i = 0; $i < @$screen_rects; $i++) {
 		my @r = @{$$cell_rects[$i]};
 		$canvas-> clipRect( @{$$screen_rects[$i]}) if $self-> {clipCells} == 2;
-		$canvas-> text_out( $self-> get_cell_text( @{$$cell_indices[$i]}), 
-			$r[0], ($r[3] + $r[1] - $font_height) / 2);
+		my ($ha, $va) = $self-> get_cell_alignment(@{$$cell_indices[$i]});
+		my ( $x, $y ) = ($r[0], $r[1]);
+		my $text = $self-> get_cell_text( @{$$cell_indices[$i]});
+		if ( $ha == ta::Right ) {
+			my $dx = $r[2] - $r[0] - $self-> get_text_width($text);
+			$x += $dx if $dx > 0;
+		} elsif ( $ha == ta::Center ) {
+			my $dx = ($r[2] - $r[0] - $self-> get_text_width($text)) / 2;
+			$x += $dx if $dx > 0;
+		}
+		if ( $va == ta::Top ) {
+			my $dy = $r[3] - $r[1] - $font_height;
+			$y += $dy if $dy > 0;
+		} elsif ( $va == ta::Middle ) {
+			my $dy = ($r[3] - $r[1] - $font_height) / 2;
+			$y += $dy if $dy > 0;
+		}
+		$canvas-> text_out_bidi( $text, $x, $y);
 	}
 	$canvas-> clipRect( @clip) if $self-> {clipCells} == 2;
 }
@@ -238,12 +264,20 @@ sub get_cell_text
 	return $txt;
 }
 
+sub get_cell_alignment
+{
+	my ( $self, $col, $row) = @_;
+	my ($ha, $va) = ( ta::Left, ta::Middle );
+	$self-> notify(q(GetAlignment), $col, $row, \$ha, \$va);
+	return $ha, $va;
+}
+
 sub get_range
 {
-	my ( $self, $axis, $index) = @_;
+	my ( $self, $vertical, $index) = @_;
 	my ( $min, $max) = ( 1, 16384 ); # actually, no real restriction on $max - 
 					# just a reasonable non-undef value
-	$self-> notify(q(GetRange), $axis, $index, \$min, \$max); 
+	$self-> notify(q(GetRange), $vertical, $index, \$min, \$max); 
 	$min = 1 if $min < 1;
 	$max = $min if $max < $min;
 	return $min, $max;
@@ -421,7 +455,7 @@ sub point2cell
 		$cy = -1;
 		$hints{y} = +1;
 	} elsif ( 
-		$self-> {lastColEmpty} && 
+		$self-> {lastRowEmpty} && 
 		$y > $a[1] + $self-> {pixelCellIndents}-> [3] &&
 		$y < $a[1] + $self-> {pixelCellIndents}-> [3] + $self-> {lastRowTail} - 
 			(($self-> {cellIndents}-> [3] > 0) ? $self-> {drawHGrid} : 0) 
@@ -798,8 +832,8 @@ sub std_draw_text_cells
 	my @colors = (
 		$self-> color,
 		$self-> backColor,
-		$self-> colorIndex( ci::HiliteText),
-		$self-> colorIndex( ci::Hilite),
+		$self-> hiliteColor,
+		$self-> hiliteBackColor,
 		$self-> indentCellColor,
 		$self-> indentCellBackColor,
 		$self-> gridColor,
@@ -808,30 +842,34 @@ sub std_draw_text_cells
 	my @f = $self-> focused ? $self-> focusedCell : ( -1, -1);
 	my @focRect;
 	my $font_height = $self-> font-> height;
-	my ( $xsel, $ysel);
+	my @prelight    = @{$self->{prelight} // [-1,-1]};
+	my ( $xsel, $ysel, $xprelight, $yprelight);
 	my ( $clipV, $clipH) = ( $self-> {clipCells} == 1, $self-> {clipCells} == 2);
 	my @clipRect = $canvas-> clipRect;
 	for ( @$cols) {
 		my ( $col, $xtype, $br, $x1, $x2, $X1, $X2) = @$_;
 		$canvas-> clipRect( $x1, $$active_area[1], $x2, $$active_area[3]) 
 			if $clipV;
-		$xsel = ( $col >= $selection[0] && $col <= $selection[2] ) ? 1 : 0;
+		$xsel      = $col >= $selection[0] && $col <= $selection[2];
+		$xprelight = $col == $prelight[0];
 		my $last_type;
 		my @bars;
 		my @rects;
 		my @cellids;
 		for ( @$rows) {
 			my ( $row, $ytype, $br, $y1, $y2, $Y1, $Y2) = @$_;
-			$ysel = ( $row >= $selection[1] && $row <= $selection[3] ) ? 1 : 0 if $xsel;
+			$ysel      = $row >= $selection[1] && $row <= $selection[3] if $xsel;
+			$yprelight = $row == $prelight[1] if $xprelight;
 			my $type = ($xtype || $ytype) ? 2 : (($xsel && $ysel) ? 1 : 0);
+			$type |= 4 if $xprelight && $yprelight;
 			if ( defined($last_type) && $type != $last_type) {
 				$canvas-> set(
-					color     => $colors[$last_type * 2],
-					backColor => $colors[$last_type * 2 + 1],
+					color     => $colors[($last_type & 3) * 2],
+					backColor => $colors[($last_type & 3) * 2 + 1],
 				);
-				$canvas-> clear(@$_) for @bars;
+				$self-> draw_item_background($canvas, @$_, $last_type & 4) for @bars;
 				$self-> draw_text_cells( $canvas, \@bars, \@rects, \@cellids, $font_height);
-				@bars = (); @rects = (); @cellids = ();
+				@bars = @rects = @cellids = ();
 			}
 			$last_type = $type;
 			push @bars,  [$x1, $y1, $x2, $y2];
@@ -842,10 +880,10 @@ sub std_draw_text_cells
 
 		if ( defined $last_type) {
 			$canvas-> set(
-				color     => $colors[$last_type * 2],
-				backColor => $colors[$last_type * 2 + 1],
+				color     => $colors[($last_type & 3) * 2],
+				backColor => $colors[($last_type & 3) * 2 + 1],
 			);
-			$canvas-> clear(@$_) for @bars;
+			$self-> draw_item_background($canvas, @$_, $last_type & 4) for @bars;
 			$self-> draw_text_cells( $canvas, \@bars, \@rects, \@cellids, $font_height);
 		}
 	}
@@ -1045,13 +1083,24 @@ sub on_mousemove
 	my ( $cx, $cy, %hints) = $self-> point2cell( $x, $y, defined($self-> {mouseTransaction}));
 	$self-> clear_event;
 	unless ( defined $self-> {mouseTransaction}) {
+		my @prelight = (-1,-1);
+		my @old      = @{$self->{prelight} // [-1,-1]};
 		if ( defined($hints{x_grid}) && $self-> allowChangeCellWidth) {
 			$self-> pointerType( cr::SizeWE);
 		} elsif ( defined($hints{y_grid}) && $self-> allowChangeCellHeight) { 
 			$self-> pointerType( cr::SizeNS);
 		} else {
+			my ( $cx, $cy, %hints) = $self-> point2cell( $x, $y);
+			@prelight = ($cx, $cy) if $hints{normal};
 			$self-> pointerType( cr::Default);
 		}
+
+		if ( join('.', @prelight ) ne join('.', @old)) {
+			$self->{prelight} = ( $prelight[0] < 0 ) ? undef : \@prelight;
+			$self->redraw_cell( @old )      if $old[0]      >= 0;
+			$self->redraw_cell( @prelight ) if $prelight[0] >= 0;
+		}
+
 		return;
 	}
 
@@ -1167,6 +1216,13 @@ sub on_mousewheel
 	$self-> topCell( $newTop);
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	my $prelight = delete $self->{prelight};
+	$self-> redraw_cell( @$prelight ) if defined $prelight;
+}
+
 sub on_paint
 {
 	my ($self,$canvas)   = @_;
@@ -1352,8 +1408,8 @@ sub on_paint
 		$$_[4] = $a[2] if $$_[4] > $a[2];
 	}
 	for ( @rowsDraw) {
-	$$_[3] = $a[1] if $$_[3] < $a[1];
-	$$_[4] = $a[3] if $$_[4] > $a[3];
+		$$_[3] = $a[1] if $$_[3] < $a[1];
+		$$_[4] = $a[3] if $$_[4] > $a[3];
 	}
 	
 	# draw cells
@@ -1647,7 +1703,7 @@ sub dy
 	}
 	$self-> scroll( 0, $delta, clipRect => \@a);
 	my @info = $self-> get_screen_cell_info( $self-> focusedCell);
-	$self-> invalidate_rect( gsci::V_RECT ) if scalar @info;
+	$self-> invalidate_rect( @info[ gsci::V_RECT ] ) if scalar @info;
 }
 
 sub focusedCell
@@ -2013,8 +2069,16 @@ sub on_getrange
 
 sub on_measure
 {
-	my ( $self, $col, $row, $sref) = @_;
-	$$sref = $self-> get_text_width( $self-> get_cell_text( $col, $row), 1);
+	my ( $self, $vertical, $index, $sref) = @_;
+	if ( $vertical) {
+		$$sref = $self-> font-> height + 2;
+	} else {
+		$$sref = 0;
+		for ( my $i = 0; $i < $self-> {colMax}; $i++ ) {
+			my $w = $self-> get_text_width( $self->get_cell_text($i, $index), 1);	
+			$$sref = $w if $$sref < $w;
+		}
+	}
 }
 
 package Prima::GridViewer;
@@ -2263,6 +2327,11 @@ sub get_cell_text
 	return $self-> {cells}-> [$row]-> [$col];
 }
 
+sub get_cell_alignment
+{
+	return (ta::Left, ta::Middle);
+}
+
 sub on_getrange
 {
 	my ( $self, $column, $index, $min, $max) = @_;
@@ -2278,11 +2347,15 @@ sub on_fontchanged
 
 sub on_measure
 {
-	my ( $self, $column, $index, $sref) = @_;
-	if ( $column) {
-		$$sref = $self-> get_text_width( $self-> {cells}-> [0]-> [$index], 1);
-	} else {
+	my ( $self, $vertical, $index, $sref) = @_;
+	if ( $vertical) {
 		$$sref = $self-> font-> height + 2;
+	} else {
+		$$sref = 0;
+		for ( @{$self-> {cells}}) {
+			my $w = $self-> get_text_width( $$_[$index], 1);	
+			$$sref = $w if $$sref < $w;
+		}
 	}
 }
 
@@ -2618,16 +2691,22 @@ Returns screen area in inclusive-inclusive pixel coordinates, that is used
 to display normal cells. The extensions are related to the current size of a widget, 
 however, can be overridden by specifying WIDTH and HEIGHT.
 
+=item get_cell_alignment COLUMN, ROW
+
+Returns two C<ta::> constants for horizontal and vertical cell text alignment.
+Since the class does not assume the item storage organization,
+the values are queried via C<GetAlignment> notification.
+
 =item get_cell_text COLUMN, ROW
 
 Returns text string assigned to cell in COLUMN and ROW.
 Since the class does not assume the item storage organization,
 the text is queried via C<Stringify> notification.
 
-=item get_range AXIS, INDEX
+=item get_range VERTICAL, INDEX
 
 Returns a pair of integers, minimal and maximal breadth of INDEXth column
-or row in pixels. If AXIS is 1, the rows are queried; if 0, the columns.
+or row in pixels. If VERTICAL is 1, the rows are queried; if 0, the columns.
 
 The method calls C<GetRange> notification.
 
@@ -2762,25 +2841,29 @@ For explanation of COLUMNS, ROWS, and AREA parameters see L<draw_cells> .
 
 =over
 
-=item DrawCell CANVAS, COLUMN, ROW, INDENT, @SCREEN_RECT, @CELL_RECT, SELECTED, FOCUSED
+=item DrawCell CANVAS, COLUMN, ROW, INDENT, @SCREEN_RECT, @CELL_RECT, SELECTED, FOCUSED, PRELIGHT
 
 Called when a cell with COLUMN and ROW coordinates is to be drawn on CANVAS. 
 SCREEN_RECT is a cell rectangle in widget coordinates,
 where the item is to be drawn. CELL_RECT is same as SCREEN_RECT, but calculated
 as if the cell is fully visible.
 
-SELECTED and FOCUSED are boolean
-flags, if the cell must be drawn correspondingly in selected and
-focused states.
+SELECTED, FOCUSED, and PRELIGHT are boolean flagss, if the cell must be drawn
+correspondingly in selected, focused, and pre-lighted states.
 
-=item GetRange AXIS, INDEX, MIN, MAX
+=item GetAlignment COLUMN, ROW, HORIZONTAL_ALIGN_REF, VERTICAL_ALIGN_REF
 
-Puts minimal and maximal breadth of INDEXth column ( AXIS = 0 ) or row ( AXIS = 1)
+Puts two text alignment C<ta::> constants, assigned to cell with COLUMN and ROW coordinates, 
+into HORIZONTAL_ALIGN_REF and VERTICAL_ALIGN_REF scalar references.
+
+=item GetRange VERTICAL, INDEX, MIN, MAX
+
+Puts minimal and maximal breadth of INDEXth column ( VERTICAL = 0 ) or row ( VERTICAL = 1)
 in corresponding MIN and MAX scalar references.
 
-=item Measure AXIS, INDEX, BREADTH
+=item Measure VERTICAL, INDEX, BREADTH
 
-Puts breadth in pixels of INDEXth column ( AXIS = 0 ) or row ( AXIS = 1)
+Puts breadth in pixels of INDEXth column ( VERTICAL = 0 ) or row ( VERTICAL = 1)
 into BREADTH scalar reference. 
 
 This notification by default may be called from within 
@@ -2791,9 +2874,9 @@ set internal flag C<{NoBulkPaintInfo}> to 1.
 
 Called when a cell with COLUMN and ROW coordinates is focused.
 
-=item SetExtent AXIS, INDEX, BREADTH 
+=item SetExtent VERTICAL, INDEX, BREADTH 
 
-Reports breadth in pixels of INDEXth column ( AXIS = 0 ) or row ( AXIS = 1),
+Reports breadth in pixels of INDEXth column ( VERTICAL = 0 ) or row ( VERTICAL = 1),
 as a response to C<columnWidth> and C<rowHeight> calls.
 
 =item Stringify COLUMN, ROW, TEXT_REF
@@ -2920,6 +3003,12 @@ its ascendants, provides a standard text grid widget.
 
 =over
 
+=item get_cell_alignment COLUMN, ROW
+
+Returns two C<ta::> constants for horizontal and vertical cell text alignment.
+Since the item storage organization is implemented, does
+so without calling C<GetAlignment> notification.
+
 =item get_cell_text COLUMN, ROW
 
 Returns text string assigned to cell in COLUMN and ROW. 
