@@ -96,7 +96,7 @@ static GTFStruct widget_types[] = {
 };
 #undef GT
 
-#if PERL_REVISION == 5 && PERL_VERSION == 20
+#if 1 || (PERL_REVISION == 5 && PERL_VERSION == 20)
 /* perl bug in 5.20.0, see more at https://rt.perl.org/Ticket/Display.html?id=122105 */
 #define PERL_PROTECT_LOCALE 1
 #endif
@@ -151,7 +151,7 @@ prima_gtk_init(void)
 			s->prima_class == wcButton
 		) ? GTK_STATE_ACTIVE : GTK_STATE_SELECTED;
 		if ( t == NULL ) {
-			warn("cannot query gtk style for %s\n", s->name);
+			Pdebug("cannot query gtk style for %s\n", s->name);
 			t = gtk_rc_get_style_by_paths(settings, NULL, NULL, GTK_TYPE_WIDGET);
 			if ( !t ) continue;
 		}
